//* -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/* Copyright (c) 2014 Adam Bieńkowski (http://launchpad.net/power-installer)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authored by: Corentin Noel <tintou@mailoo.org>
 */

// api for writing simple GUI installers

using GLib;
using Gtk;

namespace PowerInstaller {

    // all avalible strings
    private string? app_name;
    private string? category;
    private string? subtitle;
    private string? app_icon;
    private string? website;
    private string? description;
    private string? window_title;
    private string? version;

    private string? run_command_begin;
    private string? ppa;
    private string? pkg;
    private string? run_command_end;

    private string? debian_link;
    private string? theme_link;
    private string? icon_link;
    private string? plank_link;

    // info about packages (installed or not)
    private string? status;

    // display info about app or not
    private string? no_display;

    // if packages are installed display "Uninstall"
    private string  install_label;

    // labels for details window
    private string  desc_statusppa;
    private string  desc_statuspkg;

    // sensitive of buttons
    private bool    sensitive;
    private bool    installed;

    public class API : GLib.Object {

        public void init() {
            // required informations
            app_name = Environment.get_variable("APP_NAME");
            category = Environment.get_variable("CATEGORY");
            subtitle = Environment.get_variable("SUBTITLE");

            // additionals
            app_icon = Environment.get_variable("app_icon");
            website = Environment.get_variable("website");
            description = Environment.get_variable("description");
            window_title = Environment.get_variable("window_title");
            version = Environment.get_variable("version");

            // install methods
            run_command_begin = Environment.get_variable("run_command_begin");
            ppa = Environment.get_variable("ppa");
            pkg = Environment.get_variable("pkg");
            run_command_end = Environment.get_variable("run_command_end");

            // special aliases
            debian_link = Environment.get_variable("debian_link");
            theme_link = Environment.get_variable("theme_link");
            icon_link = Environment.get_variable("icon_link");
            plank_link = Environment.get_variable("plank_link");

            no_display = Environment.get_variable("no_display");

            string ALL = app_name + category + subtitle + app_icon + website + description + window_title + version + run_command_begin + ppa + pkg + run_command_end + debian_link + theme_link + icon_link + plank_link + no_display;
            if(ALL == null) {
                stdout.printf("FATAL ERROR: None of the variables were declared!\n");
                app_name = "FATAL ERROR";
                category = "NONE OF THE VARIABLES WERE DECLARED";
                subtitle = "PLEASE RUN A SHELL FILE WITH POWER INSTALLER ENTRY";
                version = description = run_command_begin = ppa = pkg = run_command_end = "";
                sensitive = false;
            } else {
                sensitive = true;
            }

            if(no_display == "") {
                no_display = "false";
            }

            // get status variable and set strings to fit in method
            status = Environment.get_variable("status");
            if(status == "installed") {
                install_label = "Uninstall";
                installed = true;
                desc_statusppa = "removed";
                desc_statuspkg = desc_statusppa;
            } else {
               install_label = "Install";
               installed = false;
               desc_statusppa = "added";
               desc_statuspkg = "installed";
            }


            if(app_name == "") { app_name = "Unknowned"; }
            if(category == "") { category = "Unknowned"; }
            if(subtitle == "") { subtitle = "Unknowned"; }

            if(app_icon == "") { app_icon = @"$DATADIR/api-icon.svg"; }
            if(window_title == "") { window_title = PROGRAM_NAME; }
            if(description == "") { description = "There is no description for this application."; }
            if(version == "") { version = "Unknown"; }

            var action_window = new ActionWindow();
            action_window.destroy.connect(Gtk.main_quit);
            action_window.create_layout(no_display, sensitive);
            action_window.show_all();
            Gtk.main();
        }
    }

    public class ActionWindow : Gtk.Window {
        private Gtk.Label app_label;
        private Gtk.Label category_label;
        private Gtk.Label subtitle_label;

        public ActionWindow() {
            this.title = "Power Installer";
            this.window_position = Gtk.WindowPosition.CENTER;
            this.border_width = 6;
            this.set_default_size(800, 200);
            this.show_all();
        }

        public void create_layout(string no_display, bool sensitive) {
            // header bar
            var header = new Gtk.HeaderBar();
            header.set_title(window_title);
            header.set_show_close_button(true);
            this.set_titlebar(header);

            // main grid
            var grid = new Gtk.EventBox();
            this.add(grid);

            // boxes
            var details = new Gtk.Box(Gtk.Orientation.VERTICAL, 0);
            var vbox = new Gtk.Box(Gtk.Orientation.VERTICAL, 0);
            var hbox = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 0);
            var info_box = new Gtk.Box(Gtk.Orientation.VERTICAL, 0);

            // pack info to vbox
            vbox.add(hbox);
            vbox.add(info_box);

            // app icon
            var icon = new Gtk.Image.from_file(app_icon);
            hbox.add(icon);

            // create category first before setting no_display key
            category_label = new Gtk.Label(@" Category: <span foreground='#c84848' size='medium'>$category</span>");
            setup_label(category_label, "h2");

            if(no_display == "true") {
                // set all to none
                app_name = category = subtitle = "";

                // remove "Category" label
                category_label.label = "";
            }


            // app name
            app_label = new Gtk.Label(@" <span foreground='#202020' font_family='Sans' size='x-large'>$app_name</span>");
            setup_label(app_label, "h1");

            // subtitle
            subtitle_label = new Gtk.Label(@" <span foreground='#9a9a9a' size='medium'>$subtitle</span>");
            subtitle_label.get_style_context().add_class("h2");
            setup_label(subtitle_label, "h2");

            // website
            var website_label = new Gtk.LinkButton.with_label(@"$website", "Official webiste");
            website_label.get_style_context().add_class("h3");
            website_label.set_alignment(0, 0);

            // pack everything to details box
            details.pack_start(app_label, false, false, 0);
            details.pack_start(category_label, false, false, 0);
            details.pack_start(subtitle_label, false, false, 0);

            // pack everything
            grid.add(vbox);
            hbox.pack_start(details, false, false, 0);

            // button box
            var button_box = new Gtk.ButtonBox(Gtk.Orientation.HORIZONTAL);

            // details button
            var details_button = new Gtk.Button.with_label("Details");
            details_button.set_sensitive(sensitive);
            details_button.clicked.connect(() => {
                var details_window = new DetailsWindow();
                details_window.show_all();
            });

            button_box.pack_start(details_button, false, false, 0);

            if(website != "") {
                button_box.add(website_label);
            }

            var install_button = new Gtk.Button.with_label(install_label);
            install_button.get_style_context().add_class("destructive-action");
            install_button.set_sensitive(sensitive);

            // execution
            install_button.clicked.connect(() => {
                var command_tab = new CommandTab();
                begin_show();
                if(@"$run_command_begin$ppa$pkg" != "") {
                    command_tab.on_go_clicked(run_command_begin, ppa, pkg, false, 'a', installed);
                }

                if(run_command_end != "") {
                    command_tab.on_go_clicked(run_command_end, "", "", false, 'a', false);
                }

                if(debian_link != "") {
                    command_tab.on_go_clicked(debian_link, "", "", true, 'b', false);
                }

                if(theme_link != "") {
                    command_tab.on_go_clicked(theme_link, "", "", true, 'c', false);
                }

                if(icon_link != "") {
                    command_tab.on_go_clicked(icon_link, "", "", true, 'd', false);
                }

                if(plank_link != "") {
                    command_tab.on_go_clicked(plank_link, "", "", true, 'e', false);
                }
            });

            button_box.pack_end(install_button, false, false, 0);
            vbox.pack_end(button_box, false, false, 0);

            grid.show_all();
        }

        private void setup_label(Gtk.Label label, string label_class) {
            label.get_style_context().add_class(label_class);
            label.set_alignment(0, 0);
            label.set_use_markup(true);
            label.set_width_chars(14);
            label.set_selectable(true);
        }
    }

    public class DetailsWindow : Gtk.Window {
        private string desc_ppa;
        private string desc_pkg;
        private string desc_run_command_begin;
        private string desc_run_command_end;

        public DetailsWindow() {
            this.title = "Details";
            this.window_position = Gtk.WindowPosition.CENTER;
            this.border_width = 6;
            this.set_default_size(500, 450);
            create_layout();
        }

        private void create_layout() {
            var header = new Gtk.HeaderBar();
            header.set_title("Details");
            header.set_show_close_button(true);
            this.set_titlebar(header);

            var main_box = new Gtk.Box(Gtk.Orientation.VERTICAL, 0);
            this.add(main_box);

            var stack = new Gtk.Stack();
            stack.set_hexpand(true);
            stack.set_transition_type(Gtk.StackTransitionType.SLIDE_LEFT);

            var stack_switcher = new Gtk.StackSwitcher();
            stack_switcher.stack = stack;
            stack_switcher.halign = Gtk.Align.CENTER;
            stack_switcher.margin_top = 2;

            //description tab
            var e_description = new Gtk.EventBox();

            var desc_box = new Gtk.Box(Gtk.Orientation.VERTICAL, 1);

            var view = new Gtk.TextView();
            view.set_wrap_mode(Gtk.WrapMode.WORD);
            view.buffer.text = @"Version: $version

$description";
            view.set_editable(false);

            var scrolled = new Gtk.ScrolledWindow(null, null);
		    scrolled.add(view);
		    desc_box.pack_start(scrolled, true, true, 0);

            // executed tab
            var executed = new Gtk.EventBox();

            var exe_box = new Gtk.Box(Gtk.Orientation.VERTICAL, 1);

            var view_exe = new Gtk.TextView();
            view_exe.set_wrap_mode(Gtk.WrapMode.WORD);

            // setup all description
            if(ppa == "") {
                desc_ppa = @"None of the repositories will be $desc_statuspkg.";
            } else {
            	desc_ppa = @"$ppa";
            }

            if(pkg == "") {
                desc_pkg = @"None of the packages will be $desc_statusppa.";
            } else {
                desc_pkg = @"$pkg";
            }

            if(run_command_begin == "") {
                desc_run_command_begin = "a) None of the beginning commands will be executed";
            } else {
                desc_run_command_begin = @"a) Beginning comands:
$run_command_begin";
            }

            if(run_command_end == "") {
                desc_run_command_end = "b) None of the closing commands will be executed";
            } else {
                desc_run_command_end = @"b) Closing commands:
$run_command_end";
            }

            view_exe.buffer.text = @"This is what will be executed in installation:

1. These repositories will be $desc_statusppa:
$desc_ppa

2. These packages will be $desc_statuspkg:
$desc_pkg

3. These commands will be executed:
$desc_run_command_begin
$desc_run_command_end

4. These files from links will be done:
$debian_link : Debian package
$icon_link : Icon theme
$plank_link : Plank theme (elementary OS)";
            view_exe.set_editable(false);

            var scrolled_exe = new Gtk.ScrolledWindow(null, null);
		    scrolled_exe.add(view_exe);
		    exe_box.pack_start(scrolled_exe, true, true, 0);

            e_description.add(desc_box);
            e_description.expand = true;

            executed.add(exe_box);
            executed.expand = true;

            stack.add_titled(e_description, "description", _("Description"));
            stack.add_titled(executed, "executed", _("Executed"));

            main_box.add(stack_switcher);
            main_box.add(stack);
        }
    }
}
